/**********************************/
/*  L Y N X  -  HexShow           */ 
/*                                */ 
/*  Matthias Domin                */
/*  6. February 1999              */
/**********************************/
 
#include <lynx.h>
#include <lynxlib.h>
#include <stdlib.h>

#define SCREEN 0xa000

#define TRUE  1
#define FALSE 0

#define JOY_RIGHT  0x10
#define JOY_LEFT   0x20
#define JOY_DOWN   0x40
#define JOY_UP     0x80

#define BUTTON_OPTION1 0x08
#define BUTTON_OPTION2 0x04
#define BUTTON_INNER   0x02
#define BUTTON_OUTER   0x01

#define BUTTON_PAUSE   0x01

struct fileentry {
  uchar StartBlock;
  uint  BlockOffset;
  uchar ExecFlag;
  char *DestAddr;
  uint  FileLen;
};

extern struct fileentry FileEntry;


char szOut[] = "CartShow by M.Domin";

uchar bMemory[1024];     // space for a whole cartridge block (1024 bytes)

/* standard palette */ 
char pal[] = {
   0x00,0x00,0x08,0x00,0x00,0x08,0x08,0x08,0x04,0x00,0x0F,0x00,0x00,0x0F,0x0F,0x0F,
   0x00,0x08,0x00,0x80,0x88,0x80,0x08,0x88,0x44,0x0F,0x00,0xF0,0xFF,0xF0,0x0F,0xFF
};

/*********************/
/* wait for a button */
/*********************/
char WaitButton()
{
   char key;
   while ( (key = joystick) == 0)
      ;
   /* wait for the button to be released */
   while ( joystick )
      ;
   return key;
}

/**************************************/
// itox2, altered version of itox
/*
   itox -- converts nbr to hex string of length sz
           right adjusted and blank filled, returns str
           if sz > 0 terminate with null byte
           if sz = 0 find end of string
           if sz < 0 use last byte for data
*/
itox2(nbr, str, sz)  
int nbr;  
char str[];  
int sz;  
{
  int digit, offset;
  if(sz>0) str[--sz]=0;
  else if(sz<0) sz = -sz;
  else while(str[sz]!=0) ++sz;
  while(sz) 
    {
    digit=nbr&15; nbr=(nbr>>4)&4095;
    if(digit<10) offset=48; else offset=55;
    str[--sz]=digit+offset;
    if(nbr==0) break;
    }
  while(sz) str[--sz]='0';  // '0' instead of blank!!!!
  return str;
}


/**************************************************************************
**                                                                       **
**                                                                       **
**************************************************************************/
main() 
{
   uint mempos;
   int xpos, ypos;
   uchar retkey;            // Has to be unsigned to check JOY_UP?
   char szHex[20+1];
   uchar bShowHex;
   uchar bChar;
   uchar bBlock, bOldBlock;


#asm
   lda #$c        /* Vector space + ROM space --> FFFA-FFFF + FE00-FFF7 */
   sta $fff9      /* MAPCTL */
   cli
#endasm

   /* set up buffer : display buffer == render buffer, no collision buffer */
   SetBuffers(SCREEN,(char *)0,(char *)0 );

   /* set the palette */
   SetRGB(pal);

   /******************************************
   *   Show the first two directory entries: *
   ******************************************/

   for (;;)  // forever
      {
      /* Clear the screen */
      DrawFBox(0, 0, 160, 102, 15);        // White


      TextOut2(8, 0, 1, 0, szOut);   // program title
/***/
      LoadCartDir(0);   // first file
      TextOut2(0, 2*8, 3, 0, "1. file: ");

      if (FileEntry.ExecFlag != 0)
         TextOut2(10*8, 2*8, 2, 0, "executable");

      itox2(FileEntry.StartBlock, szHex, 2+1);
      TextOut2(  0, 3*8, 3, 0, "Block:");
      TextOut2(6*8, 3*8, 2, 0, szHex);

      itox2(FileEntry.BlockOffset ^ 0xffff, szHex, 4+1);
      TextOut2( 9*8, 3*8, 3, 0, "Offset:");
      TextOut2(16*8, 3*8, 2, 0, szHex);

      itox2(FileEntry.FileLen ^ 0xffff, szHex, 4+1);
      TextOut2(  0, 4*8, 3, 0, "Len:");
      TextOut2(4*8, 4*8, 2, 0, szHex);

      itox2(FileEntry.DestAddr, szHex, 4+1);
      TextOut2(11*8, 4*8, 3, 0, "Addr:");
      TextOut2(16*8, 4*8, 2, 0, szHex);
/***/
      LoadCartDir(1);   // second file
      TextOut2(0, 5*8, 3, 0, "2. file: ");

      if (FileEntry.ExecFlag != 0)
         TextOut2(10*8, 5*8, 2, 0, "executable");

      itox2(FileEntry.StartBlock, szHex, 2+1);
      TextOut2(  0, 6*8, 3, 0, "Block:");
      TextOut2(6*8, 6*8, 2, 0, szHex);

      itox2(FileEntry.BlockOffset ^ 0xffff, szHex, 4+1);
      TextOut2( 9*8, 6*8, 3, 0, "Offset: ");
      TextOut2(16*8, 6*8, 2, 0, szHex);

      itox2(FileEntry.FileLen ^ 0xffff, szHex, 4+1);
      TextOut2(  0, 7*8, 3, 0, "Len:");
      TextOut2(4*8, 7*8, 2, 0, szHex);

      itox2(FileEntry.DestAddr, szHex, 4+1);
      TextOut2(11*8, 7*8, 3, 0, "Addr:");
      TextOut2(16*8, 7*8, 2, 0, szHex);
/***/
      LoadCartDir(2);   // third file
      TextOut2(0, 8*8, 3, 0, "3. file: ");

      if (FileEntry.ExecFlag != 0)
         TextOut2(10*8, 8*8, 2, 0, "executable");

      itox2(FileEntry.StartBlock, szHex, 2+1);
      TextOut2(  0, 9*8, 3, 0, "Block:");
      TextOut2(6*8, 9*8, 2, 0, szHex);

      itox2(FileEntry.BlockOffset ^ 0xffff, szHex, 4+1);
      TextOut2( 9*8, 9*8, 3, 0, "Offset: ");
      TextOut2(16*8, 9*8, 2, 0, szHex);

      itox2(FileEntry.FileLen ^ 0xffff, szHex, 4+1);
      TextOut2(  0,10*8, 3, 0, "Len:");
      TextOut2(4*8,10*8, 2, 0, szHex);

      itox2(FileEntry.DestAddr, szHex, 4+1);
      TextOut2(11*8, 10*8, 3, 0, "Addr:");
      TextOut2(16*8, 10*8, 2, 0, szHex);


      if (WaitButton() == BUTTON_INNER)
         break;
      }





   /******************************************
   * Now show the cartridge content          *
   ******************************************/

   mempos = 0;
   bBlock = 0; // first block
   bOldBlock = -1; // bBlock != bOldBlock ----> call LoadCartBlock()

for (;;)  // forever
   {
   /* Clear the screen */
   DrawFBox(0, 0, 160, 102, 15);        // White



   TextOut2(8, 0, 1, 0, szOut);   // program title

   TextOut2(0, 8, 3, 0, "Block: ");
   itox2(bBlock, szHex, 2+1);
   TextOut2(7*8, 8, 3, 0, szHex);

   TextOut2(10*8, 8, 3, 0, "Addr:");  // Address has five digits (256KB)
   itox2((bBlock / 64), szHex, 1+1);
   TextOut2(15*8, 8, 3, 0, szHex);
   itox2((bBlock % 64)*1024+mempos, szHex, 4+1);
   TextOut2(16*8, 8, 3, 0, szHex);

   TextOut2(0, 3*8, 3, 0, "Offset");

   if (bBlock != bOldBlock)
      {
      LoadCartBlock(bBlock, &bMemory[0]);
      bOldBlock = bBlock;
      }


   for (ypos = 0; ypos < 8; ypos++)
      {
      itox2(mempos+ypos*8, szHex, 4+1);        // 8 Bytes per line
      TextOut2(0, 4*8+ypos*8, 3, 0, szHex);
      for (xpos = 0; xpos < 8; xpos++)
         {
         if (bShowHex)
            {
            itox2(bMemory[mempos+ypos*8+xpos], szHex, 2+1);        // 1 Byte per cell
            TextOut2(4*8+xpos*2*8, 4*8+ypos*8, 7 + xpos % 2, 15, szHex);
            }
         else
            {
            bChar = bMemory[mempos+ypos*8+xpos];
            if (bChar < 32 || bChar > 127)
               bChar = '.';
            szHex[0] = 32;
            szHex[1] = bChar;
            szHex[2] = 0;
            TextOut2(4*8+xpos*2*8, 4*8+ypos*8, 7 + xpos % 2, 15, szHex);
            }
         }
      }

   retkey = WaitButton();
   switch (retkey)
      {
      case JOY_UP:
         if (mempos > 0)
            mempos -= 64;
         break;
      case JOY_DOWN:   // There are 8*8 bytes shown on the screen
         if (mempos < 1024-64)
            mempos += 64;
         break;

      case JOY_LEFT:
         if (bBlock > 0)
            bBlock--;
         break;
      case JOY_RIGHT:
         if (bBlock < 255)
            bBlock++;
         break;


      case BUTTON_INNER:
         bShowHex = TRUE;
         break;
      case BUTTON_OUTER:
         bShowHex = FALSE;
         break;
      }
   }
} 

